/* Copyright: Mathias Imming, 2014
 * Verwendung und Weitergabe nur unter Creative-Commons-Lizenz */
#include "CC4x.h"
#include "GPIO.h"
#include "usic.h"
#include "cmsis_os.h"

int swap_pwm;

uint8_t pwm1 = 255;
uint8_t pwm2 = 255;
uint16_t dim1 = 255;
uint16_t dim2 = 255;
uint16_t dim3 = 255;

// Counter mssen signed sein, damit -1 abgefangen werden kann
uint16_t dim1_m = 0;
int16_t dim2_m = 0;
int16_t dim3_m = 0;

uint8_t dir = 0;

const uint16_t pwmtable_10_log[256]=
{
	0,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,
	4,4,4,4,4,4,4,4,5,5,5,5,5,5,6,6,6,6,6,7,7,7,7,7,8,8,8,9,9,9,9,10,10,10,11,11,11,
	12,12,13,13,13,14,14,15,15,16,16,17,17,18,19,19,20,21,21,22,23,23,24,25,26,27,27,
	28,29,30,31,32,33,35,36,37,38,39,41,42,43,45,46,48,49,51,53,54,56,58,60,62,64,66,
	68,71,73,75,78,80,83,86,89,91,95,98,101,104,108,111,115,119,123,127,131,135,140,144,
	149,154,159,164,170,175,181,187,193,200,206,213,220,227,235,242,250,259,267,276,285,
	295,304,314,325,336,347,358,370,382,395,408,421,435,450,464,480,496,512,529,546,564,
	583,602,622,643,664,686,708,732,756,781,807,833,861,889,919,949,980,1013,1046,1081,
	1116,1153,1191,1231,1271,1313,1357,1402,1448,1496,1545,1596,1649,1703,1759,1818,1878,
	1940,2004,2070,2138,2209,2282,2357,2435,2515,2598,2684,2773,2864,2959,3057,3158,3262,
	3370,3481,3596,3715,3837,3964,4095
};


osThreadId tid_dim_task; /* Thread id of thread: phase_a */

void dim_task (void const *argument) {
	osSignalWait(0x0001, osWaitForever); /* wait for an event flag 0x0001 */
	
	for (;;) {
		osDelay(50); /* delay 50ms */
		
		if (P2_9_read()) {
			continue;
		}
		
#if 1
		if (dir & 0x01) {
			dim1_m -= 3;
			if (dim1_m <= 0) {
				dim1_m = 0;
				dir &= ~0x01;
			}
		}
		else {
			dim1_m += 3;
			if (dim1_m >= 255) {
				dim1_m = 255;
				dir |= 0x01;
			}
		}
		CCU40_CC41->CRS = PWM_RESOLTION - PWMTABLE[dim1_m]+1;
		// Slice 1 shadow transfer set enable
		CCU40->GCSS |= 0x0010;	
#endif
#if 1
		if (dir & 0x02) {
			dim2_m -= 2;
			if (dim2_m <= 0) {
				dim2_m = 0;
				dir &= ~0x02;
			}
		}
		else {
			dim2_m += 2;
			if (dim2_m >= 255) {
				dim2_m = 255;
				dir |= 0x02;
			}
		}
		CCU40_CC42->CRS = PWM_RESOLTION - PWMTABLE[dim2_m]+1;
		// Slice 1 shadow transfer set enable
		CCU40->GCSS |= 0x0100;	
#endif
#if 1
		if (dir & 0x04) {
			dim3_m -= 1;
			if (dim3_m <= 0) {
				dim3_m = 0;
				dir &= ~0x04;
			}
		}
		else {
			dim3_m += 1;
			if (dim3_m >= 255) {
				dim3_m = 255;
				dir |= 0x04;
			}
		}
		CCU40_CC43->CRS = PWM_RESOLTION - PWMTABLE[dim3_m]+1;
		// Slice 1 shadow transfer set enable
		CCU40->GCSS |= 0x1000;	
#endif		
	}
}

osThreadDef(dim_task, osPriorityNormal, 1, 0);


void CCU40_0_IRQHandler(void)
{
	if (swap_pwm == 0)
	{
		// Compare Value fr Channel 1 setzen
		// Out Port tauschen
		P0_5_set_mode(OUTPUT_PP_GP);
		P0_6_set_mode(OUTPUT_PP_AF4);
		CCU40_CC40->CRS = 15350+(14*pwm1);
	}
	else
	{
		// Compare Value fr Channel 2 setzen
		// Out Port tauschen
		P0_5_set_mode(OUTPUT_PP_AF4);
		P0_6_set_mode(OUTPUT_PP_GP);
		CCU40_CC40->CRS = 15600+(10*pwm2);
	}
	swap_pwm = !swap_pwm;
	
	// Slice 0 shadow transfer set enable
	CCU40->GCSS |= 0x0001;	
}

void CC4_Init(void) {	
	// Enable the CCU4 clock via the specific SCU register, CGATCLR0
	SCU_GENERAL->PASSWD = 0x000000C0UL;
	SCU_CLK->CGATCLR0 |= 0x04;
	SCU_GENERAL->PASSWD = 0x000000C3UL;
	
	// Enable the prescaler block
	CCU40->GIDLC |= 0x0100;
	
	CC40_CC40_Init();
	CC40_CC41_Init();
	CC40_CC42_Init();
	CC40_CC43_Init();
	
	tid_dim_task = osThreadCreate(osThread(dim_task), NULL);
	osSignalSet(tid_dim_task, 0x0001); /* send signal to phaseA thread */
}

void CC40_CC40_Init(void) {
	//Prescaler F CCU4 / 16 = 32Mhz / 16 = 2 Mhz
	CCU40_CC40->PSC = 0x00000004UL;

	// Period Value
	CCU40_CC40->PRS = 20000; 		// ergibt bei 2Mhz einen Timer-Takt von 10ms: 1/2.000.000 [s] * 20.000 = 0,01s = 10ms
	// Compare Value
	CCU40_CC40->CRS = 18000;		// ergibt einen INT bzw. Toggle des Pins nach 9ms
	
	// Slice 0 shadow transfer set enable
	CCU40->GCSS |= 0x0001;
	
  // Remove Idle Mode from Channel	
	CCU40->GIDLC |= 0x01;
	
	// Period match interrupt on SR0
	CCU40_CC40->INTE |= 0x01;
	CCU40_CC41->SRS |= 0x00;
	
	//----INTERRUPT-SETUP-------------------------------------------------------------------------------
	NVIC_ClearPendingIRQ(CCU40_0_IRQn);		//This function clears node 1 pending status
	NVIC_EnableIRQ(CCU40_0_IRQn);			//This function enables node 1 for interrupt request generation
	
	// Timer clear
	CCU40_CC40->TCCLR |= 0x02;
	// start Timer
	CCU40_CC40->TCSET |= 0x01;
	
	// Port 0.5 und 0.6 abwechselnd aus PWM-Out nutzen -> ALT4
	P0_5_set_mode(OUTPUT_PP_AF4);
	P0_6_set_mode(OUTPUT_PP_GP);
}

void CC40_CC41_Init(void) {
	//Prescaler F CCU4 / 0 = 32Mhz / 0 = 32 Mhz
	CCU40_CC41->PSC = 0x00000000UL;

	// Period Value
	CCU40_CC41->PRS = PWM_RESOLTION;
	// Compare Value
	CCU40_CC41->CRS = PWM_RESOLTION - PWMTABLE[dim1_m] +1;
	
	// Slice 1 shadow transfer set enable
	CCU40->GCSS |= 0x0010;
	
  // Remove Idle Mode from Channel	
	CCU40->GIDLC |= 0x02;
	
	// Period match interrupt on SR1
	CCU40_CC41->INTE |= 0x01;
	CCU40_CC41->SRS |= 0x01;
		
	// Timer clear
	CCU40_CC41->TCCLR |= 0x02;
	// start Timer
	CCU40_CC41->TCSET |= 0x01;
	
	// Port 0.7  aus PWM-Out nutzen -> ALT4
	P0_7_set_mode(OUTPUT_PP_AF4);
}

void CC40_CC42_Init(void) {
	//Prescaler F CCU4 / 0 = 32Mhz / 0 = 32 Mhz
	CCU40_CC42->PSC = 0x00000000UL;

	// Period Value
	CCU40_CC42->PRS = PWM_RESOLTION;
	// Compare Value
	CCU40_CC42->CRS = PWM_RESOLTION - PWMTABLE[dim2_m] +1;;
	
	// Slice 2 shadow transfer set enable
	CCU40->GCSS |= 0x0100;
	
  // Remove Idle Mode from Channel	
	CCU40->GIDLC |= 0x04;
	
	// Period match interrupt on SR2
	CCU40_CC42->INTE |= 0x01;
	CCU40_CC42->SRS |= 0x02;
		
	// Timer clear
	CCU40_CC42->TCCLR |= 0x02;
	// start Timer
	CCU40_CC42->TCSET |= 0x01;
	
	// Port 0.8  aus PWM-Out nutzen -> ALT4
	P0_8_set_mode(OUTPUT_PP_AF4);
}


void CC40_CC43_Init(void) {
	//Prescaler F CCU4 / 0 = 32Mhz / 0 = 32 Mhz
	CCU40_CC43->PSC = 0x00000000UL;

	// Period Value
	CCU40_CC43->PRS = PWM_RESOLTION;
	// Compare Value
	CCU40_CC43->CRS = PWM_RESOLTION - PWMTABLE[dim3_m] +1;;
	
	// Slice 3 shadow transfer set enable
	CCU40->GCSS |= 0x1000;
	
  // Remove Idle Mode from Channel	
	CCU40->GIDLC |= 0x08;
	
	// Period match interrupt on SR3
	CCU40_CC43->INTE |= 0x01;
	CCU40_CC43->SRS |= 0x03;
	
	// Timer clear
	CCU40_CC43->TCCLR |= 0x02;
	// start Timer
	CCU40_CC43->TCSET |= 0x01;
	
	// Port 0.9  aus PWM-Out nutzen -> ALT4
	P0_9_set_mode(OUTPUT_PP_AF4);
}
