/* Copyright: Mathias Imming, 2014
 * Verwendung und Weitergabe nur unter Creative-Commons-Lizenz 
 * Initialisierungs-Routine frei nach Uwe Becker (http://mikrocontroller.bplaced.net/)
 */
#include "usic.h"
#include "GPIO.h"
#include "CC4x.h"

uint8_t dmx_rbuf[513];
volatile int16_t dmx_rx_cnt = 0;
uint8_t dmx_okcnt;
uint32_t fifo_remain;
uint32_t rbr;

uint8_t dmx_lpi;		/* Lichtprotokoll Merkbit 			*/
uint8_t dmx_lp;		/* Lichtprotokoll					*/

uint32_t uart_remain;
uint8_t uart_rbuf[40];
uint8_t uart_cnt = 0;

void USIC_INIT(void) {
	// Enable the USIC clock via the specific SCU register, CGATCLR0
	SCU_GENERAL->PASSWD = 0x000000C0UL;
	SCU_CLK->CGATCLR0 |= 0x08;
	SCU_GENERAL->PASSWD = 0x000000C3UL;
	
	// DMX
	USIC0_CH1_Init();
	
	//UART
	USIC0_CH0_Init();
}

void USIC0_1_IRQHandler(void) {
	uint32_t receive_word;
	
	// denn kompletten Fifo lesen
	while (USIC0_CH1->TRBSR & USIC_CH_TRBSR_RBFLVL_Msk)
	{
		receive_word = USIC0_CH1->OUTR;
			
		/* ist es das Byte nach dem Startcode? */
		if (dmx_rx_cnt == 1)
		{				
			switch (dmx_rbuf[0])
			{
				case STARTCODE_LICHT:
					/* Letztes Protokoll auf Lichtcode testen */
					if (dmx_okcnt++ < DMX_OK_CNT) {
						dmx_rx_cnt = DMX_MSG_LEN + 1;			// folgende Bytes werden ignoriert bis neuer Break erkannt wird
					}		
					break;
				
				// TODO: andere Startcodes, die genormt sind, ignorieren
				default:				// andere Startcodes
					dmx_okcnt = 0;
					dmx_rx_cnt = DMX_MSG_LEN + 1;
					break;
			}
		}
		
		/* Bytes ab 0 in Puffer eintragen
		 * alles ueber 512 wird ignoriert
		 * die 3-5 Kreise: LTP-Merge (Latest take precentage)
		 * PWM fr Servos wird ber CC-Int abgehandelt
		 */
		if (dmx_rx_cnt <= DMX_MSG_LEN)
		{				
			if (dmx_rbuf[dmx_rx_cnt] != (receive_word &0xFF)) {			
				switch (dmx_rx_cnt) {
					case 1:
						pwm1 = receive_word & 0xFF;
						break;
					case 2:
						pwm2 = receive_word & 0xFF;
						break;
					case 3:
						CCU40_CC41->CRS = PWM_RESOLTION - PWMTABLE[(receive_word & 0xFF)]+1;
						CCU40->GCSS |= 0x0010;	
						break;
					case 4:
						CCU40_CC42->CRS = PWM_RESOLTION - PWMTABLE[(receive_word & 0xFF)]+1;
						CCU40->GCSS |= 0x0100;	
						break;
					case 5:
						CCU40_CC43->CRS = PWM_RESOLTION - PWMTABLE[(receive_word & 0xFF)]+1;
						CCU40->GCSS |= 0x1000;	
						break;
					default:
						break;
				}
			}
			dmx_rbuf[dmx_rx_cnt++] = (receive_word & 0xFF);
		}	
	}
	
	USIC0_CH1->TRBSCR |= (0x01 << USIC_CH_TRBSCR_CSRBI_Pos); 				// clear Standard receive buffer event

	/* Break nach FIFO auswerten, damit Rest-Bytes noch abgeholt werden */	
	if (USIC0_CH1->PSR & USIC_CH_PSR_ASCMode_SBD_Msk)				// Break
	{
		dmx_rx_cnt = 0;				// Zaehler auf 0
		USIC0_CH1->PSCR |= USIC_CH_PSCR_CST2_Msk | USIC_CH_PSCR_CST5_Msk | USIC_CH_PSCR_CST6_Msk | USIC_CH_PSCR_CST7_Msk | USIC_CH_PSCR_CRSIF_Msk | USIC_CH_PSCR_CRIF_Msk;
	}
	else if (USIC0_CH1->PSR & (USIC_CH_PSR_ASCMode_FER0_Msk | USIC_CH_PSR_ASCMode_FER1_Msk))			// Stop-Bit Error
	{	
		//dmx_okcnt = dmx_lp = dmx_lpi = dmx_rdm_st0 = 0;
		dmx_rx_cnt = 0;				// Zaehler auf 0
		USIC0_CH1->PSCR |= USIC_CH_PSCR_CST2_Msk | USIC_CH_PSCR_CST5_Msk | USIC_CH_PSCR_CST6_Msk | USIC_CH_PSCR_CST7_Msk | USIC_CH_PSCR_CRSIF_Msk | USIC_CH_PSCR_CRIF_Msk; 	// 0x44E4
	}
}

void USIC0_CH1_Init(void){
	// stop USIC_CH1
	USIC0_CH1->CCR = 0;
	
	/* Enable the module kernel clock and the module functionality  */
	USIC0_CH1->KSCFG |= USIC_CH_KSCFG_MODEN_Msk | USIC_CH_KSCFG_BPMODEN_Msk;
	/* fFD = fPB */
  /* FDR.DM = 2 (Fractional divider mode) */
  USIC0_CH1->FDR &= ~(USIC_CH_FDR_DM_Msk | USIC_CH_FDR_STEP_Msk);
  USIC0_CH1->FDR |= (0x02UL << USIC_CH_FDR_DM_Pos) | (DMX_FDR_STEP << USIC_CH_FDR_STEP_Pos);

  /* Configure baud rate generator */
  /* BAUDRATE = fCTQIN/(BRG.PCTQ x BRG.DCTQ) */
  /* CLKSEL = 0 (fPIN = fFD), CTQSEL = 00b (fCTQIN = fPDIV), PPPEN = 0 (fPPP=fPIN) */
  USIC0_CH1->BRG &= ~(USIC_CH_BRG_PCTQ_Msk | USIC_CH_BRG_DCTQ_Msk | USIC_CH_BRG_PDIV_Msk | USIC_CH_BRG_CLKSEL_Msk | USIC_CH_BRG_PPPEN_Msk);
  USIC0_CH1->BRG |= (DMX_BRG_PCTQ << USIC_CH_BRG_PCTQ_Pos) | (DMX_BRG_DCTQ << USIC_CH_BRG_DCTQ_Pos) | (DMX_BRG_PDIV << USIC_CH_BRG_PDIV_Pos);

	/* Configuration of USIC Shift Control */
  /* SCTR.FLE = 8 (Frame Length) */
  /* SCTR.WLE = 8 (Word Length) */
  /* SCTR.TRM = 1 (Transmission Mode) */
  /* SCTR.PDL = 1 (This bit defines the output level at the shift data output signal when no data is available for transmission) */ 
  USIC0_CH1->SCTR &= ~(USIC_CH_SCTR_TRM_Msk | USIC_CH_SCTR_FLE_Msk | USIC_CH_SCTR_WLE_Msk);
  USIC0_CH1->SCTR |= USIC_CH_SCTR_PDL_Msk | (0x01UL << USIC_CH_SCTR_TRM_Pos) | (0x07UL << USIC_CH_SCTR_FLE_Pos) | (0x07UL << USIC_CH_SCTR_WLE_Pos); 

  /* Configuration of USIC Transmit Control/Status Register */
  /* TBUF.TDEN = 1 (TBUF Data Enable: A transmission of the data word in TBUF can be started if TDV = 1 */
  /* TBUF.TDSSM = 1 (Data Single Shot Mode: allow word-by-word data transmission which avoid sending the same data several times*/
  USIC0_CH1->TCSR &= ~(USIC_CH_TCSR_TDEN_Msk);
  USIC0_CH1->TCSR |= USIC_CH_TCSR_TDSSM_Msk | (0x01UL << USIC_CH_TCSR_TDEN_Pos);

  /* Configuration of Protocol Control Register			 			*/
  /* PCR.SMD = 1 (Sample Mode based on majority) 					*/
  /* PCR.STPB = 1 (2x Stop bit) 													*/
  /* PCR.SP = 4 (Sample Point) 														*/
  /* PCR.PL = 0 (Pulse Length is equal to the bit length) */
	/* PCR.SBIEN = 1 (Break Interrupt enable								*/
  USIC0_CH1->PCR &= ~(USIC_CH_PCR_ASCMode_STPB_Msk | USIC_CH_PCR_ASCMode_SP_Msk | USIC_CH_PCR_ASCMode_PL_Msk);
	USIC0_CH1->PCR |= (0x01 << USIC_CH_PCR_ASCMode_STPB_Pos) | (0x01 << USIC_CH_PCR_ASCMode_SBIEN_Pos); 
  USIC0_CH1->PCR |=  USIC_CH_PCR_ASCMode_SMD_Msk | (4 << USIC_CH_PCR_ASCMode_SP_Pos);

  /* Configure Transmit Buffer */
  /* Standard transmit buffer event is enabled */
  /* Define start entry of Transmit Data FIFO buffer DPTR = 0*/
  USIC0_CH1->TBCTR &= ~(USIC_CH_TBCTR_SIZE_Msk | USIC_CH_TBCTR_DPTR_Msk);

  /* Set Transmit Data Buffer to 16 and set data pointer to position 0*/
  USIC0_CH1->TBCTR |= ((0x04 << USIC_CH_TBCTR_SIZE_Pos)|(0x00 << USIC_CH_TBCTR_DPTR_Pos));

  /* Initialize DMX_RX (DX0)*/
	P2_10_enable_digital();
	P2_10_set_mode(INPUT);
	
	//DX0F: DSEL = 0x05
	USIC0_CH1->DX0CR &= ~(USIC_CH_DX0CR_DSEL_Msk);
	USIC0_CH1->DX0CR |= (0x05 << USIC_CH_DX0CR_DSEL_Pos);

  /* Configure Receive Buffer */
  /* Standard Receive buffer event is enabled */
  /* Define start entry of Receive Data FIFO buffer DPTR = 16*/
  USIC0_CH1->RBCTR &= ~(USIC_CH_RBCTR_SIZE_Msk | USIC_CH_RBCTR_DPTR_Msk);

  /* Set Receive Data Buffer Size to 16 and set data pointer to position 16*/
  USIC0_CH1->RBCTR |= (16 << USIC_CH_RBCTR_DPTR_Pos);
	USIC0_CH1->RBCTR |= (0x04UL << USIC_CH_RBCTR_SIZE_Pos);
	USIC0_CH1->RBCTR |=  (0x01 << USIC_CH_RBCTR_SRBIEN_Pos) | (0x03UL << USIC_CH_RBCTR_RCIM_Pos);
	// Point to SR1
	USIC0_CH1->RBCTR |= (0x01 << USIC_CH_RBCTR_SRBINP_Pos) | (0x01 << USIC_CH_RBCTR_ARBINP_Pos);
	// Point Break Int to SR1
	USIC0_CH1->INPR |= (0x01  << USIC_CH_INPR_PINP_Pos);

  /* Configuration of Channel Control Register */
  /* CCR.PM = 00 ( Disable parity generation) */
  /* CCR.MODE = 2 (ASC mode enabled. Note: 0 (USIC channel is disabled)) */
  USIC0_CH1->CCR &= ~(USIC_CH_CCR_PM_Msk | USIC_CH_CCR_MODE_Msk);
  USIC0_CH1->CCR |= (0x02UL << USIC_CH_CCR_MODE_Pos) | (0x01 << USIC_CH_CCR_RIEN_Pos) | (0x01 << USIC_CH_CCR_AIEN_Pos);
	
	/* INTERRUPT-SETUP 	*/
	NVIC_ClearPendingIRQ(USIC0_1_IRQn);		//This function clears node 1 pending status
	NVIC_EnableIRQ(USIC0_1_IRQn);					//This function enables node 1 for interrupt request generation
}


//-------- PC UART Handling ---------------------------------------------------------------------------

void USIC0_0_IRQHandler(void) {
	// solange Receive Buffer nicht leer ist
	while (((USIC0_CH0->TRBSR & USIC_CH_TRBSR_RBFLVL_Msk) >> USIC_CH_TRBSR_RBFLVL_Pos) > 0)
	{		
		if (uart_cnt < 40) {
			uart_rbuf[uart_cnt++] = USIC0_CH0->OUTR & 0xFF;
		}
		else {
			uart_cnt = 50;
			break;
		}
		
		if (uart_cnt >= 3) {
			if (uart_rbuf[2] != 0xAD)
				return;
			
			switch (uart_rbuf[0]) {
				case 0x10:
					pwm1 = uart_rbuf[1];
					break;
				case 0x20:
					pwm2 = uart_rbuf[1];
					break;
				case 0x30:
					CCU40_CC41->CRS = PWM_RESOLTION - PWMTABLE[uart_rbuf[1]]+1;
					CCU40->GCSS |= 0x0010;	
					break;
				case 0x40:
					CCU40_CC42->CRS = PWM_RESOLTION - PWMTABLE[uart_rbuf[1]]+1;
					CCU40->GCSS |= 0x0100;	
					break;
				case 0x50:
					CCU40_CC43->CRS = PWM_RESOLTION - PWMTABLE[uart_rbuf[1]]+1;
					CCU40->GCSS |= 0x1000;	
					break;
				default:
				break;
			}
			uart_cnt = 0;
		}
	}
}

void USIC0_CH0_Init(void){
	// auf Digital schalten
  P2_1_enable_digital();
  P2_2_enable_digital();	
	
	// stop USIC_CH0
	USIC0_CH0->CCR = 0;
	
	/* Enable the module kernel clock and the module functionality  */
	USIC0_CH0->KSCFG |= USIC_CH_KSCFG_MODEN_Msk | USIC_CH_KSCFG_BPMODEN_Msk;
	/* fFD = fPB */
  /* FDR.DM = 02b (Fractional divider mode) */
  USIC0_CH0->FDR &= ~(USIC_CH_FDR_DM_Msk | USIC_CH_FDR_STEP_Msk);
  USIC0_CH0->FDR |= (0x02UL << USIC_CH_FDR_DM_Pos) | (UART_FDR_STEP << USIC_CH_FDR_STEP_Pos);

  /* Configure baud rate generator */
  /* BAUDRATE = fCTQIN/(BRG.PCTQ x BRG.DCTQ) */
  /* CLKSEL = 0 (fPIN = fFD), CTQSEL = 00b (fCTQIN = fPDIV), PPPEN = 0 (fPPP=fPIN) */
  USIC0_CH0->BRG &= ~(USIC_CH_BRG_PCTQ_Msk | USIC_CH_BRG_DCTQ_Msk | USIC_CH_BRG_PDIV_Msk | USIC_CH_BRG_CLKSEL_Msk | USIC_CH_BRG_PPPEN_Msk);
  USIC0_CH0->BRG |= (UART_BRG_PCTQ << USIC_CH_BRG_PCTQ_Pos) | (UART_BRG_DCTQ << USIC_CH_BRG_DCTQ_Pos) | (UART_BRG_PDIV << USIC_CH_BRG_PDIV_Pos);

	/* Configuration of USIC Shift Control */
  /* SCTR.FLE = 8 (Frame Length) */
  /* SCTR.WLE = 8 (Word Length) */
  /* SCTR.TRM = 1 (Transmission Mode) */
  /* SCTR.PDL = 1 (This bit defines the output level at the shift data output signal when no data is available for transmission) */ 
  USIC0_CH0->SCTR &= ~(USIC_CH_SCTR_TRM_Msk | USIC_CH_SCTR_FLE_Msk | USIC_CH_SCTR_WLE_Msk);
  USIC0_CH0->SCTR |= USIC_CH_SCTR_PDL_Msk | (0x01UL << USIC_CH_SCTR_TRM_Pos) | (0x07UL << USIC_CH_SCTR_FLE_Pos) | (0x07UL << USIC_CH_SCTR_WLE_Pos); 

  /* Configuration of USIC Transmit Control/Status Register */
  /* TBUF.TDEN = 1 (TBUF Data Enable: A transmission of the data word in TBUF can be started if TDV = 1 */
  /* TBUF.TDSSM = 1 (Data Single Shot Mode: allow word-by-word data transmission which avoid sending the same data several times*/
  USIC0_CH0->TCSR &= ~(USIC_CH_TCSR_TDEN_Msk);
  USIC0_CH0->TCSR |= USIC_CH_TCSR_TDSSM_Msk | (0x01UL << USIC_CH_TCSR_TDEN_Pos);

  /* Configuration of Protocol Control Register */
  /* PCR.SMD = 1 (Sample Mode based on majority) */
  /* PCR.STPB = 0 (2x Stop bit) */
  /* PCR.SP = 4 (Sample Point) */
  /* PCR.PL = 0 (Pulse Length is equal to the bit length) */
  USIC0_CH0->PCR &= ~(USIC_CH_PCR_ASCMode_STPB_Msk | USIC_CH_PCR_ASCMode_SP_Msk | USIC_CH_PCR_ASCMode_PL_Msk);
  USIC0_CH0->PCR |=  USIC_CH_PCR_ASCMode_SMD_Msk | (4 << USIC_CH_PCR_ASCMode_SP_Pos);

  /* Configure Transmit Buffer */
  /* Standard transmit buffer event is enabled */
  /* Define start entry of Transmit Data FIFO buffer DPTR = 0*/
  USIC0_CH0->TBCTR &= ~(USIC_CH_TBCTR_SIZE_Msk | USIC_CH_TBCTR_DPTR_Msk);

  /* Set Transmit Data Buffer to 16 and set data pointer to position 32*/
  USIC0_CH0->TBCTR |= ((0x04 << USIC_CH_TBCTR_SIZE_Pos)|(0x32 << USIC_CH_TBCTR_DPTR_Pos));

  /* Initialize UART_RX (DX0)*/
  /* P2.2 as input */
  P2_2_set_mode(INPUT);

  /* Select P2.2 as input for USIC DX3 -> UCIC DX0 */
  USIC0_CH0->DX3CR &= ~(USIC_CH_DX3CR_DSEL_Msk);

  /* Route USIC DX3 input signal to USIC DX0 (DSEL=DX0G) */
  USIC0_CH0->DX0CR &= ~(USIC_CH_DX0CR_DSEL_Msk);
  USIC0_CH0->DX0CR |= 6 << USIC_CH_DX0CR_DSEL_Pos;

  /* Configure Receive Buffer */
  /* Standard Receive buffer event is enabled */
  /* Define start entry of Receive Data FIFO buffer DPTR = 16*/
  USIC0_CH0->RBCTR &= ~(USIC_CH_RBCTR_SIZE_Msk | USIC_CH_RBCTR_DPTR_Msk | USIC_CH_RBCTR_LIMIT_Msk);

  /* Set Receive Data Buffer Size to 16 and set data pointer to position 48*/
  USIC0_CH0->RBCTR |= (48 << USIC_CH_RBCTR_DPTR_Pos);
	USIC0_CH0->RBCTR |= (0x04UL << USIC_CH_RBCTR_SIZE_Pos);
	USIC0_CH0->RBCTR |=  (0x01 << USIC_CH_RBCTR_SRBIEN_Pos) | (0x03UL << USIC_CH_RBCTR_RCIM_Pos);
	USIC0_CH0->RBCTR |= (0x01 << USIC_CH_RBCTR_LIMIT_Pos) | (0x01 << USIC_CH_RBCTR_SRBIEN_Pos) | (0x01 << USIC_CH_RBCTR_ARBIEN_Pos);

  /* Initialize UART_TX (DOUT)*/
  /* P2.1 as output controlled by ALT6 = U0C0.DOUT0 */
  P2_1_set_mode(OUTPUT_PP_AF6);

  /* Configuration of Channel Control Register */
  /* CCR.PM = 00 ( Disable parity generation) */
  /* CCR.MODE = 2 (ASC mode enabled. Note: 0 (USIC channel is disabled)) */
  USIC0_CH0->CCR &= ~(USIC_CH_CCR_PM_Msk | USIC_CH_CCR_MODE_Msk);
  USIC0_CH0->CCR |= (0x02UL << USIC_CH_CCR_MODE_Pos) | (0x01 << USIC_CH_CCR_RIEN_Pos) | (0x01 << USIC_CH_CCR_AIEN_Pos);
	
	/* INTERRUPT-SETUP 	*/
	NVIC_ClearPendingIRQ(USIC0_0_IRQn);
	NVIC_EnableIRQ(USIC0_0_IRQn);
}
